import json
import os
import sys
from multiprocessing import Process
import time

from urllib2 import HTTPError
from test.temboard import init_env, drop_env, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)

from temboardagent.spc import connector, error  # noqa

ENV = {}
XSESSION = ''


def pg_sleep(duration=1):
    """
    Start a new PG connection and run pg_sleep()
    """
    conn = connector(
        host=ENV['pg']['socket_dir'],
        port=ENV['pg']['port'],
        user=ENV['pg']['user'],
        password=ENV['pg']['password'],
        database='postgres'
        )
    try:
        conn.connect()
        conn.execute("SELECT pg_sleep(%s)" % (duration))
        conn.close()
    except error:
        pass


def create_database(dbname):
    """
    Create a database.
    """
    conn = connector(
        host=ENV['pg']['socket_dir'],
        port=ENV['pg']['port'],
        user=ENV['pg']['user'],
        password=ENV['pg']['password'],
        database='postgres'
        )
    try:
        conn.connect()
        conn.execute("CREATE DATABASE %s" % (dbname))
        conn.close()
    except error:
        pass


def create_table(dbname, tablename):
    """
    Create a table and insert 5 rows in it.
    """
    conn = connector(
        host=ENV['pg']['socket_dir'],
        port=ENV['pg']['port'],
        user=ENV['pg']['user'],
        password=ENV['pg']['password'],
        database=dbname
        )
    try:
        conn.connect()
        conn.execute("CREATE TABLE %s (id INTEGER)" % (tablename))
        conn.execute("INSERT INTO %s SELECT generate_series(1, 5)" % (
                        tablename
                        ))
        conn.close()
    except error:
        pass


def lock_table_exclusive(dbname, tablename, duration):
    """
    Lock a table in exclusive mode for a while (duration in seconds).
    """
    conn = connector(
        host=ENV['pg']['socket_dir'],
        port=ENV['pg']['port'],
        user=ENV['pg']['user'],
        password=ENV['pg']['password'],
        database=dbname
        )
    try:
        conn.connect()
        conn.execute("BEGIN")
        conn.execute("LOCK TABLE %s IN EXCLUSIVE MODE" % (tablename))
        time.sleep(float(duration))
        conn.execute("ROLLBACK")
        conn.close()
    except error:
        pass


def update_rows(dbname, tablename):
    """
    Update all rows of a table.
    """
    conn = connector(
        host=ENV['pg']['socket_dir'],
        port=ENV['pg']['port'],
        user=ENV['pg']['user'],
        password=ENV['pg']['password'],
        database=dbname
        )
    try:
        conn.connect()
        conn.execute("UPDATE %s SET id = id + 1" % (tablename))
        conn.close()
    except error:
        pass


class TestActivity:

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def _exec_query(self, dbname, query):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database=dbname
        )
        conn.connect()
        conn.execute(query)
        conn.close()
        return list(conn.get_rows())

    def _temboard_login(self):
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        return json.loads(res)['session']

    def test_00_env_pg(self):
        """
        [activity] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            global XSESSION
            XSESSION = self._temboard_login()
            assert True
        except error:
            assert False

    def test_01_activity_root(self):
        """
        [activity] 01: GET /activity : Check HTTP code (200) and the whole data structure
        """  # noqa

        # Start a long query in a dedicated process.
        p = Process(target=pg_sleep, args=(1,))
        p.start()

        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/activity' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        dict_data = json.loads(res)

        # Get the number of backends
        nb_backends = self._exec_query(
                        'postgres',
                        "SELECT COUNT(pid) AS nb FROM pg_stat_activity "
                        "WHERE pid != pg_backend_pid()")[0]['nb']

        # Join the process
        p.join()

        assert status == 200
        assert 'rows' in dict_data
        assert type(dict_data['rows']) == list
        # Check if the number of rows is equal to the number of backends
        assert len(dict_data['rows']) == nb_backends
        assert type(dict_data['rows'][0]) == dict
        assert 'pid' in dict_data['rows'][0]
        assert 'database' in dict_data['rows'][0]
        assert 'user' in dict_data['rows'][0]
        assert 'client' in dict_data['rows'][0]
        assert 'cpu' in dict_data['rows'][0]
        assert 'memory' in dict_data['rows'][0]
        assert 'read_s' in dict_data['rows'][0]
        assert 'write_s' in dict_data['rows'][0]
        assert 'iow' in dict_data['rows'][0]
        assert 'wait' in dict_data['rows'][0]
        assert 'duration' in dict_data['rows'][0]
        assert 'state' in dict_data['rows'][0]
        assert 'query' in dict_data['rows'][0]
        assert type(dict_data['rows'][0]['pid']) == int
        assert type(dict_data['rows'][0]['database']) == unicode
        assert type(dict_data['rows'][0]['user']) == unicode
        assert type(dict_data['rows'][0]['cpu']) == float
        assert type(dict_data['rows'][0]['memory']) == float
        assert type(dict_data['rows'][0]['read_s']) == unicode
        assert type(dict_data['rows'][0]['write_s']) == unicode
        assert type(dict_data['rows'][0]['iow']) == unicode
        assert type(dict_data['rows'][0]['wait']) == unicode
        assert type(dict_data['rows'][0]['duration']) == float
        assert type(dict_data['rows'][0]['state']) == unicode
        assert type(dict_data['rows'][0]['query']) == unicode

    def test_02_activity_kill(self):
        """
        [activity] 02: POST /activity/kill : Test backend termination
        """  # noqa

        # Start a very long query in a dedicated process.
        p = Process(target=pg_sleep, args=(3,))
        p.start()

        # Sleep a while before inspecting pg_stat_activity...
        time.sleep(1)

        # Get the pid of the backend running the long query
        backend_pid = self._exec_query(
                        'postgres',
                        "SELECT pid FROM pg_stat_activity "
                        "WHERE query LIKE 'SELECT pg\_sleep%'")[0]['pid']

        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/activity/kill' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    "pids": [backend_pid]
                }
            )
        except HTTPError as e:
            status = e.code
        dict_data = json.loads(res)

        # Join the process
        p.join()

        assert status == 200
        assert 'backends' in dict_data
        assert type(dict_data['backends']) == list
        assert type(dict_data['backends'][0]) == dict
        assert 'pid' in dict_data['backends'][0]
        assert 'killed' in dict_data['backends'][0]
        assert type(dict_data['backends'][0]['pid']) == int
        assert type(dict_data['backends'][0]['killed']) == bool
        assert dict_data['backends'][0]['pid'] == backend_pid

    def test_03_activity_waiting(self):
        """
        [activity] 03: GET /activity/waiting : Check HTTP code (200) and the whole data structure
        """  # noqa

        # Create a new DB
        dbname = 'testdbw'
        tablename = 'testtablew'
        create_database(dbname)
        create_table(dbname, tablename)
        # Lock table in exclusive mode in a dedicated process for 3 seconds.
        p1 = Process(target=lock_table_exclusive, args=(dbname, tablename, 3))
        p1.start()
        # Sleep a bit
        time.sleep(1)
        # Try to update rows from the locked table
        p2 = Process(target=update_rows, args=(dbname, tablename))
        p2.start()

        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/activity/waiting' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        dict_data = json.loads(res)

        # Join the process
        p1.join()
        p2.join()

        assert status == 200
        assert 'rows' in dict_data
        assert type(dict_data['rows']) == list
        # One row is expected
        assert len(dict_data['rows']) == 1
        assert type(dict_data['rows'][0]) == dict
        assert 'pid' in dict_data['rows'][0]
        assert 'database' in dict_data['rows'][0]
        assert 'user' in dict_data['rows'][0]
        assert 'cpu' in dict_data['rows'][0]
        assert 'memory' in dict_data['rows'][0]
        assert 'read_s' in dict_data['rows'][0]
        assert 'write_s' in dict_data['rows'][0]
        assert 'iow' in dict_data['rows'][0]
        assert 'relation' in dict_data['rows'][0]
        assert 'type' in dict_data['rows'][0]
        assert 'mode' in dict_data['rows'][0]
        assert 'duration' in dict_data['rows'][0]
        assert 'state' in dict_data['rows'][0]
        assert 'query' in dict_data['rows'][0]
        assert type(dict_data['rows'][0]['pid']) == int
        assert type(dict_data['rows'][0]['database']) == unicode
        assert type(dict_data['rows'][0]['user']) == unicode
        assert type(dict_data['rows'][0]['cpu']) == float
        assert type(dict_data['rows'][0]['memory']) == float
        assert type(dict_data['rows'][0]['read_s']) == unicode
        assert type(dict_data['rows'][0]['write_s']) == unicode
        assert type(dict_data['rows'][0]['iow']) == unicode
        assert type(dict_data['rows'][0]['relation']) == unicode
        assert type(dict_data['rows'][0]['type']) == unicode
        assert dict_data['rows'][0]['type'] == u'relation'
        assert type(dict_data['rows'][0]['mode']) == unicode
        assert dict_data['rows'][0]['mode'] == u'RowExclusiveLock'
        assert type(dict_data['rows'][0]['duration']) == float
        assert type(dict_data['rows'][0]['state']) == unicode
        assert type(dict_data['rows'][0]['query']) == unicode

    def test_04_activity_blocking(self):
        """
        [activity] 04: GET /activity/blocking : Check HTTP code (200) and the whole data structure
        """  # noqa

        # Create a new DB
        dbname = 'testdbb'
        tablename = 'testtableb'
        create_database(dbname)
        create_table(dbname, tablename)
        # Lock table in exclusive mode in a dedicated process for 3 seconds.
        p1 = Process(target=lock_table_exclusive, args=(dbname, tablename, 3))
        p1.start()
        # Sleep a bit
        time.sleep(1)
        # Try to update rows from the locked table
        p2 = Process(target=update_rows, args=(dbname, tablename))
        p2.start()

        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/activity/blocking' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        dict_data = json.loads(res)

        # Join the process
        p1.join()
        p2.join()

        assert status == 200
        assert 'rows' in dict_data
        assert type(dict_data['rows']) == list
        # One row is expected
        assert len(dict_data['rows']) == 1
        assert type(dict_data['rows'][0]) == dict
        assert 'pid' in dict_data['rows'][0]
        assert 'database' in dict_data['rows'][0]
        assert 'user' in dict_data['rows'][0]
        assert 'cpu' in dict_data['rows'][0]
        assert 'memory' in dict_data['rows'][0]
        assert 'read_s' in dict_data['rows'][0]
        assert 'write_s' in dict_data['rows'][0]
        assert 'iow' in dict_data['rows'][0]
        assert 'relation' in dict_data['rows'][0]
        assert 'type' in dict_data['rows'][0]
        assert 'mode' in dict_data['rows'][0]
        assert 'duration' in dict_data['rows'][0]
        assert 'state' in dict_data['rows'][0]
        assert 'query' in dict_data['rows'][0]
        assert type(dict_data['rows'][0]['pid']) == int
        assert type(dict_data['rows'][0]['database']) == unicode
        assert type(dict_data['rows'][0]['user']) == unicode
        assert type(dict_data['rows'][0]['cpu']) == float
        assert type(dict_data['rows'][0]['memory']) == float
        assert type(dict_data['rows'][0]['read_s']) == unicode
        assert type(dict_data['rows'][0]['write_s']) == unicode
        assert type(dict_data['rows'][0]['iow']) == unicode
        assert type(dict_data['rows'][0]['relation']) == unicode
        assert type(dict_data['rows'][0]['type']) == unicode
        assert dict_data['rows'][0]['type'] == u'relation'
        assert type(dict_data['rows'][0]['mode']) == unicode
        assert dict_data['rows'][0]['mode'] == u'ExclusiveLock'
        assert type(dict_data['rows'][0]['duration']) == float
        assert type(dict_data['rows'][0]['state']) == unicode
        assert type(dict_data['rows'][0]['query']) == unicode



import json
import time
import os
import sys

from urllib2 import HTTPError
from test.temboard import init_env, drop_env, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)


from temboardagent.spc import connector, error  # noqa

ENV = {}
XSESSION = ''


class TestAdministration:

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def _create_dummy_db(self, dbname):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        conn.connect()
        conn.execute("CREATE DATABASE %s" % (dbname))
        conn.close()

    def _drop_dummy_db(self, dbname):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        conn.connect()
        conn.execute("DROP DATABASE %s" % (dbname))
        conn.close()

    def _create_dummy_table(self, dbname, tablename):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database=dbname
        )
        conn.connect()
        conn.execute("CREATE TABLE %s (id INTEGER)" % (tablename))
        conn.execute("INSERT INTO %s SELECT generate_series(1, 500000)" % (
                        tablename
                        ))
        conn.close()

    def _get_duration_since_last_vacuum(self, dbname, tablename):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database=dbname
        )
        conn.connect()
        query = """
SELECT
    coalesce(EXTRACT(EPOCH FROM (NOW() - last_vacuum)),0) AS duration
FROM
    pg_stat_user_tables
WHERE
    relname = '%s'""" % (
            tablename
            )
        conn.execute(query)
        conn.close()
        return list(conn.get_rows())[0]['duration']

    def _temboard_login(self):
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        return json.loads(res)['session']

    def test_00_env_pg(self):
        """
        [administration] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            global XSESSION
            XSESSION = self._temboard_login()
            assert True
        except error:
            assert False

    def test_01_vacuum_ok(self):
        """
        [administration] 01: POST /administration/vacuum : Schedule & run a VACUUM on a table
        """  # noqa
        dummy_dbname = 'test_temboard'
        dummy_tablename = 'test_vacuum'

        self._create_dummy_db(dummy_dbname)
        self._create_dummy_table(dummy_dbname, dummy_tablename)

        (status, res) = temboard_request(
            ENV['agent']['ssl_cert_file'],
            method='POST',
            url='https://%s:%s/administration/vacuum' % (
                    ENV['agent']['host'],
                    ENV['agent']['port']
                    ),
            headers={
                "Content-type": "application/json",
                "X-Session": XSESSION
            },
            data={
                "database": dummy_dbname,
                "table": dummy_tablename,
                "mode": "standard"
            }
        )

        if status != 200:
            self._drop_dummy_db(dummy_dbname)
            assert False
        else:
            time.sleep(2)
            if self._get_duration_since_last_vacuum(
                    dummy_dbname, dummy_tablename) > 0:
                self._drop_dummy_db(dummy_dbname)
                assert True
            else:
                self._drop_dummy_db(dummy_dbname)
                assert False

    def test_02_vacuum_uniqness(self):
        """
        [administration] 02: POST /administration/vacuum : Schedule & run 2 VACUUM orders on the same table, getting a 402 error is expected on the 2nd call
        """  # noqa
        dummy_dbname = 'test_temboard02'
        dummy_tablename = 'test_vacuum02'

        self._create_dummy_db(dummy_dbname)
        self._create_dummy_table(dummy_dbname, dummy_tablename)
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/administration/vacuum' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    "database": dummy_dbname,
                    "table": dummy_tablename,
                    "mode": "standard"
                }
            )
        except HTTPError as e:
            status = e.code
        if status != 200:
            self._drop_dummy_db(dummy_dbname)
            assert False
        else:
            status = 0
            try:
                (status, res) = temboard_request(
                    ENV['agent']['ssl_cert_file'],
                    method='POST',
                    url='https://%s:%s/administration/vacuum' % (
                            ENV['agent']['host'],
                            ENV['agent']['port']
                            ),
                    headers={
                        "Content-type": "application/json",
                        "X-Session": XSESSION
                    },
                    data={
                        "database": dummy_dbname,
                        "table": dummy_tablename,
                        "mode": "standard"
                    }
                )
            except HTTPError as e:
                status = e.code
            self._drop_dummy_db(dummy_dbname)
            assert status == 402

    def test_03_command_status(self):
        """
        [administration] 03: GET /command/<cid> : Schedule a VACUUM order, and check status
        """  # noqa
        dummy_dbname = 'test_temboard'
        dummy_tablename = 'test_vacuum'

        self._create_dummy_db(dummy_dbname)
        self._create_dummy_table(dummy_dbname, dummy_tablename)
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/administration/vacuum' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    "database": dummy_dbname,
                    "table": dummy_tablename,
                    "mode": "standard"
                }
            )
        except HTTPError as e:
            status = e.code

        if status != 200:
            self._drop_dummy_db(dummy_dbname)
            assert False
        # First call to /command/<cid>, command's state must be set to 0
        # (work's not been started yet), or 1 (work's been started)
        cid = json.loads(res)['cid']
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/command/%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        cid
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code

        if status != 200:
            self._drop_dummy_db(dummy_dbname)
            assert False
        d_data = json.loads(res)
        if d_data['state'] != 0 and d_data['state'] != 1:
            self._drop_dummy_db(dummy_dbname)
            assert False
        # Sleep for 5 seconds, after that vacuum worker should be done.
        time.sleep(5)
        # Then let do a 2nd call, command's state must be set to 2
        # (COMMAND_DONE)
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/command/%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        cid
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        if status != 200:
            self._drop_dummy_db(dummy_dbname)
            assert False
        d_data = json.loads(res)
        if d_data['state'] != 2:
            self._drop_dummy_db(dummy_dbname)
            assert False
        # On this last call, command status shouldn't be returned
        # because the command has been removed after the last call to the API
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/command/%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        cid
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        self._drop_dummy_db(dummy_dbname)
        assert status == 401



import sys
import traceback
import socket
from logging import (Logger, Formatter, DEBUG, INFO, WARNING, ERROR, CRITICAL,
                     FileHandler, StreamHandler)
from logging.handlers import SysLogHandler
from temboardagent.errors import ConfigurationError


"""

Logging things should follow this template:

[INFO] "We are going to do something"
[DEBUG] Raw data, when we're working with data
if error:
    [DEBUG] Error backtrace
    [ERROR] Error message from the exception
    [INFO] "Failed."
else:
    [INFO] "Done."

"""


def get_tb():
    exc_info = sys.exc_info()
    lines = traceback.format_exc().splitlines()
    del exc_info
    return lines


# Mapping configuration -> constants
LOG_FACILITIES = {
    'local0': SysLogHandler.LOG_LOCAL0,
    'local1': SysLogHandler.LOG_LOCAL1,
    'local2': SysLogHandler.LOG_LOCAL2,
    'local3': SysLogHandler.LOG_LOCAL3,
    'local4': SysLogHandler.LOG_LOCAL4,
    'local5': SysLogHandler.LOG_LOCAL5,
    'local6': SysLogHandler.LOG_LOCAL6,
    'local7': SysLogHandler.LOG_LOCAL7
}

LOG_LEVELS = {
    'DEBUG': DEBUG,
    'INFO': INFO,
    'WARNING': WARNING,
    'ERROR': ERROR,
    'CRITICAL': CRITICAL
}

LOG_METHODS = ['syslog', 'file', 'stderr']


class Log(Logger):
    """
    Logger
    """
    _instances = dict()

    def __new__(cls, config, name, *args, **kwargs):
        # Create a new logger only once.
        if name not in cls._instances:
            inst = super(Log, cls).__new__(cls)
            cls._instances[name] = inst
        return cls._instances[name]

    def __init__(self, config, name, *args, **kwargs):
        Logger.__init__(self, name, *args, **kwargs)

        log_format = "temboard-agent[%(process)d]: [%(name)s] %(levelname)s:"\
                     " %(message)s"

        if config.logging['method'] == 'syslog':
            try:
                # Instanciate a new syslog handler.
                lh = SysLogHandler(
                        address=str(config.logging['destination']),
                        facility=LOG_FACILITIES[config.logging['facility']])
            except socket.error as e:
                raise ConfigurationError(e)
        elif config.logging['method'] == 'file':
            try:
                # Instanciate a new file handler.
                lh = FileHandler(
                        filename=config.logging['destination'],
                        mode='a')
                # Add timestamp when using a FileHandler.
                log_format = "%(asctime)s "+log_format
            except IOError as e:
                raise ConfigurationError(e)
        elif config.logging['method'] == 'stderr':
            lh = StreamHandler()
            # Add timestamp
            log_format = "%(asctime)s "+log_format

        # Set log level according to the level defined in configuration files.
        lh.setLevel(LOG_LEVELS[config.logging['level']])
        lh.setFormatter(Formatter(log_format))
        self.addHandler(lh)

    def traceback(self, tb_lines):
        if isinstance(tb_lines, list):
            for l in tb_lines:
                self.debug(l)
        else:
            self.debug(tb_lines)


LOGGER_NAME = 'temboard-agent'


def set_logger_name(name):
    global LOGGER_NAME
    LOGGER_NAME = name


def get_logger(config):
    """
    Returns a logger instance.
    """
    return Log(config, LOGGER_NAME)



import json
import os
import sys

from test.temboard import init_env, drop_env, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)

from temboardagent.spc import connector, error  # noqa

ENV = {}
XSESSION = ''


class TestDashboard:

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def _temboard_login(self):
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        return json.loads(res)['session']

    def test_00_env_pg(self):
        """
        [dashboard] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            global XSESSION
            XSESSION = self._temboard_login()
            assert True
        except error:
            assert False

    def test_01_dashboard_ok(self):
        """
        [dashboard] 01: GET /dashboard : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        dict_data = json.loads(res)
        assert status == 200 \
            and 'active_backends' in dict_data \
            and 'loadaverage' in dict_data \
            and 'os_version' in dict_data \
            and 'pg_version' in dict_data \
            and 'n_cpu' in dict_data \
            and 'hitratio' in dict_data \
            and 'databases' in dict_data \
            and 'memory' in dict_data \
            and 'hostname' in dict_data \
            and 'cpu' in dict_data \
            and 'buffers' in dict_data

    def test_02_dashboard_buffers_ok(self):
        """
        [dashboard] 02: GET /dashboard/buffers : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/buffers' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        dict_data = json.loads(res)
        assert status == 200 \
            and 'buffers' in dict_data \
            and 'nb' in dict_data['buffers'] \
            and type(dict_data['buffers']['nb']) == int \
            and 'time' in dict_data['buffers'] \
            and type(dict_data['buffers']['time']) == float

    def test_03_dashboard_hitratio_ok(self):
        """
        [dashboard] 03: GET /dashboard/hitratio : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/hitratio' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        dict_data = json.loads(res)
        assert status == 200 \
            and 'hitratio' in dict_data \
            and type(dict_data['hitratio']) == float

    def test_04_dashboard_active_backends_ok(self):
        """
        [dashboard] 04: GET /dashboard/active_backends : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/active_backends' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        dict_data = json.loads(res)
        assert status == 200 \
            and 'active_backends' in dict_data \
            and 'nb' in dict_data['active_backends'] \
            and type(dict_data['active_backends']['nb']) == int \
            and 'time' in dict_data['active_backends'] \
            and type(dict_data['active_backends']['time']) == float

    def test_05_dashboard_cpu_ok(self):
        """
        [dashboard] 05: GET /dashboard/cpu : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/cpu' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        dict_data = json.loads(res)
        assert status == 200 \
            and 'cpu' in dict_data \
            and 'iowait' in dict_data['cpu'] \
            and type(dict_data['cpu']['iowait']) == float \
            and dict_data['cpu']['iowait'] >= 0 \
            and dict_data['cpu']['iowait'] <= 100 \
            and 'idle' in dict_data['cpu'] \
            and type(dict_data['cpu']['idle']) == float \
            and dict_data['cpu']['idle'] >= 0 \
            and dict_data['cpu']['idle'] <= 100 \
            and 'steal' in dict_data['cpu'] \
            and type(dict_data['cpu']['steal']) == float \
            and dict_data['cpu']['steal'] >= 0 \
            and dict_data['cpu']['steal'] <= 100 \
            and 'user' in dict_data['cpu'] \
            and type(dict_data['cpu']['user']) == float \
            and dict_data['cpu']['user'] >= 0 \
            and dict_data['cpu']['user'] <= 100 \
            and 'system' in dict_data['cpu'] \
            and type(dict_data['cpu']['system']) == float \
            and dict_data['cpu']['system'] >= 0 \
            and dict_data['cpu']['system'] <= 100 \
            and dict_data['cpu']['iowait'] + \
                dict_data['cpu']['idle'] + \
                dict_data['cpu']['steal'] + \
                dict_data['cpu']['user'] + \
                dict_data['cpu']['system'] >= 99.5 \
            and dict_data['cpu']['iowait'] + \
                dict_data['cpu']['idle'] + \
                dict_data['cpu']['steal'] + \
                dict_data['cpu']['user'] + \
                dict_data['cpu']['system'] <= 100.5

    def test_06_dashboard_loadaverage_ok(self):
        """
        [dashboard] 06: GET /dashboard/loadaverage : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/loadaverage' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'loadaverage' in dict_data \
            and type(dict_data['loadaverage']) == float

    def test_07_dashboard_memory_ok(self):
        """
        [dashboard] 07: GET /dashboard/memory : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/memory' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'memory' in dict_data \
            and 'total' in dict_data['memory'] \
            and type(dict_data['memory']['total']) == int \
            and 'active' in dict_data['memory'] \
            and type(dict_data['memory']['active']) == float \
            and dict_data['memory']['active'] >= 0 \
            and dict_data['memory']['active'] <= 100 \
            and 'cached' in dict_data['memory'] \
            and type(dict_data['memory']['cached']) == float \
            and dict_data['memory']['cached'] >= 0 \
            and dict_data['memory']['cached'] <= 100 \
            and 'free' in dict_data['memory'] \
            and type(dict_data['memory']['free']) == float \
            and dict_data['memory']['free'] >= 0 \
            and dict_data['memory']['free'] <= 100 \
            and dict_data['memory']['active'] + \
                dict_data['memory']['cached'] + \
                dict_data['memory']['free'] >= 99.5 \
            and dict_data['memory']['active'] + \
                dict_data['memory']['cached'] + \
                dict_data['memory']['free'] <= 100.5

    def test_08_dashboard_hostname_ok(self):
        """
        [dashboard] 08: GET /dashboard/hostname : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/hostname' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'hostname' in dict_data \
            and type(dict_data['hostname']) == unicode

    def test_09_dashboard_os_version_ok(self):
        """
        [dashboard] 09: GET /dashboard/os_version : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/os_version' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'os_version' in dict_data \
            and type(dict_data['os_version']) == unicode

    def test_10_dashboard_databases_ok(self):
        """
        [dashboard] 10: GET /dashboard/databases : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/databases' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'databases' in dict_data \
            and 'total_size' in dict_data['databases'] \
            and type(dict_data['databases']['total_size']) == unicode \
            and 'time' in dict_data['databases'] \
            and type(dict_data['databases']['time']) == unicode \
            and 'databases' in dict_data['databases'] \
            and type(dict_data['databases']['databases']) == int \
            and dict_data['databases']['databases'] >= 0 \
            and 'total_commit' in dict_data['databases'] \
            and type(dict_data['databases']['total_commit']) == int \
            and 'total_rollback' in dict_data['databases'] \
            and type(dict_data['databases']['total_rollback']) == int

    def test_11_dashboard_pg_version_ok(self):
        """
        [dashboard] 11: GET /dashboard/pg_version : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/pg_version' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'pg_version' in dict_data \
            and type(dict_data['pg_version']) == unicode

    def test_12_dashboard_n_cpu_ok(self):
        """
        [dashboard] 12: GET /dashboard/n_cpu : HTTP return code is 200 and the data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/dashboard/n_cpu' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200 \
            and 'n_cpu' in dict_data \
            and type(dict_data['n_cpu']) == int



import json
import re
import os
import sys

from urllib2 import HTTPError
from test.temboard import init_env, drop_env, rand_string, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)

from temboardagent.spc import connector, error  # noqa

ENV = {}


class TestAPI:

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def test_00_env_pg(self):
        """
        [api] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            assert True
        except error:
            assert False

    def test_01_login_ok(self):
        """
        [api] 01: POST /login : HTTP return code is 200 on valid credentials
        """
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        assert status == 200

    def test_02_login_ok(self):
        """
        [api] 02: POST /login : Session ID format matches ^[a-f0-9]{64}$
        """
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        xsession = json.loads(res)['session']
        assert re.match('^[a-f0-9]{64}$', xsession)

    def test_03_login_ko(self):
        """
        [api] 03: POST /login : Wrong password must return 404 HTTP code
        """
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': rand_string(12)
                    }
                )
        except HTTPError as e:
            status = e.code
        assert status == 404

    def test_04_login_ko(self):
        """
        [api] 04: POST /login : Wrong username must return 404 HTTP code
        """
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': rand_string(12),
                    'password': rand_string(12)
                    }
                )
        except HTTPError as e:
            status = e.code
        assert status == 404

    def test_05_login_ko(self):
        """
        [api] 05: POST /login : Bad username format must return 406 HTTP code
        """
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': rand_string(2),
                    'password': rand_string(12)
                    }
                )
        except HTTPError as e:
            status = e.code
        assert status == 406

    def test_06_login_ko(self):
        """
        [api] 06: POST /login : Bad password format must return 406 HTTP code
        """
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': rand_string(12),
                    'password': rand_string(2)
                    }
                )
        except HTTPError as e:
            status = e.code
        assert status == 406



try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import os
import json
import re
from temboardagent.errors import ConfigurationError
from temboardagent.logger import LOG_FACILITIES, LOG_LEVELS, LOG_METHODS


class BaseConfiguration(configparser.RawConfigParser):
    """
    Common configuration parser.
    """
    def __init__(self, configfile, *args, **kwargs):
        configparser.RawConfigParser.__init__(self, *args, **kwargs)
        self.configfile = configfile
        # Default configuration values
        self.temboard = {
            'port': 2345,
            'address': '0.0.0.0',
            'users': '/etc/temboard-agent/users',
            'ssl_cert_file': None,
            'ssl_key_file': None,
            'plugins': [
                "monitoring",
                "dashboard",
                "pgconf",
                "administration",
                "activity"
            ],
            'home': '/var/lib/temboard-agent',
            'hostname': None,
            'key': None
        }
        self.logging = {
            'method': 'syslog',
            'facility': 'local0',
            'destination': '/dev/log',
            'level': 'INFO'
        }
        self.postgresql = {
            'host': '/var/run/postgresql',
            'user': 'postgres',
            'port': 5432,
            'password': None,
            'dbname': 'postgres',
            'pg_config': '/usr/bin/pg_config',
            'instance': 'main'
        }
        try:
            with open(self.configfile) as fd:
                self.readfp(fd)
        except IOError:
            raise ConfigurationError("Configuration file %s can't be opened."
                                     % (self.configfile))
        except configparser.MissingSectionHeaderError:
            raise ConfigurationError(
                    "Configuration file does not contain section headers.")

    def check_section(self, section):
        if not self.has_section(section):
            raise ConfigurationError(
                    "Section '%s' not found in configuration file %s"
                    % (section, self.configfile))


class Configuration(BaseConfiguration):
    """
    Customized configuration parser.
    """
    def __init__(self, configfile, *args, **kwargs):
        BaseConfiguration.__init__(self, configfile, *args, **kwargs)
        self.plugins = {}
        # Test if 'temboard' section exists.
        self.check_section('temboard')

        try:
            if not (self.getint('temboard', 'port') >= 0
                    and self.getint('temboard', 'port') <= 65535):
                raise ValueError()
            self.temboard['port'] = self.getint('temboard', 'port')
        except ValueError:
            raise ConfigurationError("'port' option must be an integer "
                                     "[0-65535] in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass
        try:
            if not re.match(r'(?:[3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}|\d)'
                            '(\.(?:[3-9]\d?|2(?:5[0-5]|[0-4]?\d)?|1\d{0,2}|\d'
                            ')){3}$', self.get('temboard', 'address')):
                raise ValueError()
            self.temboard['address'] = self.get('temboard', 'address')
        except ValueError:
            raise ConfigurationError("'address' option must be a valid IPv4 "
                                     "address in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass

        try:
            self.temboard['users'] = self.get('temboard', 'users')
        except configparser.NoOptionError:
            pass

        try:
            plugins = json.loads(self.get('temboard', 'plugins'))
            if not type(plugins) == list:
                raise ValueError()
            for plugin in plugins:
                if not re.match('^[a-zA-Z0-9]+$', str(plugin)):
                    raise ValueError
            self.temboard['plugins'] = plugins
        except configparser.NoOptionError:
            pass
        except ValueError:
            raise ConfigurationError("'plugins' option must be a list of "
                                     "string (alphanum only) in %s." % (
                                         self.configfile))
        try:
            self.temboard['key'] = self.get('temboard', 'key')
        except configparser.NoOptionError:
            pass

        try:
            with open(self.get('temboard', 'ssl_cert_file')) as fd:
                fd.read()
                self.temboard['ssl_cert_file'] = self.get('temboard',
                                                          'ssl_cert_file')
        except Exception:
            raise ConfigurationError("SSL certificate file %s can't be opened."
                                     % (self.get('temboard', 'ssl_cert_file')))
        except configparser.NoOptionError:
            pass

        try:
            with open(self.get('temboard', 'ssl_key_file')) as fd:
                fd.read()
                self.temboard['ssl_key_file'] = self.get('temboard',
                                                         'ssl_key_file')
        except Exception:
            raise ConfigurationError("SSL private key file %s can't be opened."
                                     % (self.get('temboard', 'ssl_key_file')))
        except configparser.NoOptionError:
            pass

        try:
            home = self.get('temboard', 'home')
            if not os.access(home, os.W_OK):
                raise Exception()
            self.temboard['home'] = self.get('temboard', 'home')
        except Exception:
            raise ConfigurationError("Home directory %s not writable."
                                     % (self.get('temboard', 'home')))
        except configparser.NoOptionError:
            pass

        try:
            hostname = self.get('temboard', 'hostname')
            self.temboard['hostname'] = hostname
        except configparser.NoOptionError:
            pass

        # Test if 'logging' section exists.
        self.check_section('logging')
        try:
            if not self.get('logging', 'method') in LOG_METHODS:
                raise ValueError()
            self.logging['method'] = self.get('logging', 'method')
        except ValueError:
            raise ConfigurationError("Invalid 'method' option in 'logging' "
                                     "section in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass
        try:
            if not self.get('logging', 'facility') in LOG_FACILITIES:
                raise ValueError()
            self.logging['facility'] = self.get('logging', 'facility')
        except ValueError:
            raise ConfigurationError("Invalid 'facility' option in 'logging' "
                                     "section in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass
        try:
            self.logging['destination'] = self.get('logging', 'destination')
        except configparser.NoOptionError:
            pass
        try:
            if not self.get('logging', 'level') in LOG_LEVELS:
                raise ValueError()
            self.logging['level'] = self.get('logging', 'level')
        except ValueError:
            raise ConfigurationError("Invalid 'level' option in 'logging' "
                                     "section in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass

        # Test if 'postgresql' section exists.
        self.check_section('postgresql')
        try:
            from os import path
            if not path.exists(self.get('postgresql', 'host')):
                raise ValueError()
            self.postgresql['host'] = self.get('postgresql', 'host')
        except ValueError:
            raise ConfigurationError("'host' option must be a valid directory"
                                     " path containing PostgreSQL's local unix"
                                     " socket in %s." % (self.configfile))
        except configparser.NoOptionError:
            pass

        try:
            self.postgresql['user'] = self.get('postgresql', 'user')
        except configparser.NoOptionError:
            pass

        try:
            if not (self.getint('postgresql', 'port') >= 0
                    and self.getint('postgresql', 'port') <= 65535):
                raise ValueError()
            self.postgresql['port'] = self.getint('postgresql', 'port')
        except ValueError:
            raise ConfigurationError("'port' option must be an integer "
                                     "[0-65535] in 'postgresql' section in %s."
                                     % (self.configfile))
        except configparser.NoOptionError:
            pass

        try:
            self.postgresql['password'] = self.get('postgresql', 'password')
        except configparser.NoOptionError:
            pass

        try:
            self.postgresql['dbname'] = self.get('postgresql', 'dbname')
        except configparser.NoOptionError:
            pass
        try:
            self.postgresql['instance'] = self.get('postgresql', 'instance')
        except configparser.NoOptionError:
            pass


class PluginConfiguration(configparser.RawConfigParser):
    """
    Customized configuration parser for plugins.
    """
    def __init__(self, configfile, *args, **kwargs):
        configparser.RawConfigParser.__init__(self, *args, **kwargs)
        self.configfile = configfile

        try:
            with open(self.configfile) as fd:
                self.readfp(fd)
        except IOError:
            raise ConfigurationError("Configuration file %s can't be opened."
                                     % (self.configfile))
        except configparser.MissingSectionHeaderError:
            raise ConfigurationError("Configuration file does not contain "
                                     "section headers.")

    def check_section(self, section):
        if not self.has_section(section):
            raise ConfigurationError("Section '%s' not found in configuration "
                                     "file %s" % (section, self.configfile))


class LazyConfiguration(BaseConfiguration):
    """
    Customized configuration parser.
    """
    def __init__(self, configfile, *args, **kwargs):
        BaseConfiguration.__init__(self, configfile, *args, **kwargs)
        # Test if 'temboard' section exists.
        self.check_section('temboard')
        for k, v in self.temboard.iteritems():
            try:
                self.temboard[k] = self.get('temboard', k)
            except configparser.NoOptionError:
                pass
        # Test if 'logging' section exists.
        self.check_section('logging')
        for k, v in self.logging.iteritems():
            try:
                self.logging[k] = self.get('logging', k)
            except configparser.NoOptionError:
                pass
        # Test if 'postgresql' section exists.
        self.check_section('postgresql')
        for k, v in self.logging.iteritems():
            try:
                self.postgresql[k] = self.get('postgresql', k)
            except configparser.NoOptionError:
                pass



from optparse import OptionParser


class CLIOptions(OptionParser):
    """
    Command line interface options parser.
    """
    def __init__(self, *args, **kwargs):
        OptionParser.__init__(self, *args, **kwargs)
        self.add_option("-c",
                        "--config",
                        dest="configfile",
                        help="Configuration file. Default: %default",
                        default="/etc/temboard-agent/temboard-agent.conf")


class temboardOptions(CLIOptions):
    """
    temboard-agent options parser.
    """
    def __init__(self, *args, **kwargs):
        CLIOptions.__init__(self, *args, **kwargs)
        self.add_option("-d",
                        "--daemon",
                        dest="daemon",
                        action="store_true",
                        help="Run in background. Default: %default",
                        default=False)
        self.add_option("-p",
                        "--pid-file",
                        dest="pidfile",
                        help="PID file. Default: %default",
                        default="/run/temboard-agent.pid")


class agentRegisterOptions(CLIOptions):
    """
    Command line interface options parser.
    """
    def __init__(self, *args, **kwargs):
        CLIOptions.__init__(self, *args, **kwargs)
        self.add_option("-h",
                        "--host",
                        dest="host",
                        help="Agent address. Default: %default",
                        default="localhost")
        self.add_option("-p",
                        "--port",
                        dest="port",
                        help="Agent listening TCP port. Default: %default",
                        default="2345")
        self.add_option("-g",
                        "--groups",
                        dest="groups",
                        help="Instance groups list, comma separated. "
                             "Default: %default",
                        default=None)
        self.add_option("--help",
                        dest="help",
                        action="store_true",
                        help="Show this help message and exit.",
                        default=False)



import json
import os
import sys

from urllib2 import HTTPError
from test.temboard import init_env, drop_env, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)

from temboardagent.spc import connector, error  # noqa

ENV = {}
XSESSION = ''


class TestMonitoring:

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def _temboard_login(self):
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        return json.loads(res)['session']

    def test_00_env_pg(self):
        """
        [administration] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            global XSESSION
            XSESSION = self._temboard_login()
            assert True
        except error:
            assert False

    def test_01_monitoring_session(self):
        """
        [monitoring] 01: GET /monitoring/probe/sessions : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/sessions' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_02_monitoring_xacts(self):
        """
        [monitoring] 02: GET /monitoring/probe/xacts : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/xacts' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_03_monitoring_locks(self):
        """
        [monitoring] 03: GET /monitoring/probe/locks : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/locks' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_04_monitoring_blocks(self):
        """
        [monitoring] 04: GET /monitoring/probe/blocks : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/blocks' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_05_monitoring_bgwriter(self):
        """
        [monitoring] 05: GET /monitoring/probe/bgwriter : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/bgwriter' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_06_monitoring_db_size(self):
        """
        [monitoring] 06: GET /monitoring/probe/db_size : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/db_size' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_07_monitoring_tblspc_size(self):
        """
        [monitoring] 07: GET /monitoring/probe/tblspc_size : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/tblspc_size' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_08_monitoring_filesystems_size(self):
        """
        [monitoring] 08: GET /monitoring/probe/filesystems_size : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/filesystems_size' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_09_monitoring_cpu(self):
        """
        [monitoring] 09: GET /monitoring/probe/cpu : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/cpu' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_10_monitoring_process(self):
        """
        [monitoring] 10: GET /monitoring/probe/process : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/process' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_11_monitoring_memory(self):
        """
        [monitoring] 11: GET /monitoring/probe/memory : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/memory' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_12_monitoring_loadavg(self):
        """
        [monitoring] 12: GET /monitoring/probe/loadavg : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/loadavg' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_13_monitoring_wal_files(self):
        """
        [monitoring] 13: GET /monitoring/probe/wal_files : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/wal_files' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200

    def test_14_monitoring_replication(self):
        """
        [monitoring] 14: GET /monitoring/probe/replication : Check HTTP code returned is 200
        """  # noqa
        status = 0
        try:
            (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/monitoring/probe/replication' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                        ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        except HTTPError as e:
            status = e.code
        assert status == 200



from setuptools import setup, find_packages
import subprocess


try:
    # pip install mode
    with open('PKG-INFO') as fo:
        for line in fo:
            if not line.startswith('Version: '):
                continue
            VERSION = line.replace('Version: ', '').strip()
            break
except IOError:
    try:
        # Release mode
        # git describe returns version[-count-gsha1].
        version, count, sha = (
            subprocess.check_output(["git", "describe", "--tags"])
            .strip().decode() + '--'
        ).split('-', 3)[:3]
    except Exception:
        VERSION = '0'
    else:
        VERSION = version
        if count:
            VERSION += '.dev%s' % (count,)

setup(
    name='temboard-agent',
    version=VERSION,
    author='Dalibo',
    author_email='contact@dalibo.com',
    license='PostgreSQL',
    url='http://temboard.io/',
    description='Administration & monitoring PostgreSQL agent.',
    long_description=open('README.rst').read(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: No Input/Output (Daemon)",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 2 :: Only",
        "Topic :: Database :: Database Engines/Servers",
        "Topic :: System :: Monitoring",
    ],
    packages=find_packages(),
    scripts=[
        'temboard-agent',
        'temboard-agent-adduser',
        'temboard-agent-password',
        'temboard-agent-register',
    ],
    data_files=[
        ('share/temboard-agent/', [
            'share/temboard-agent.conf.sample',
            'share/temboard-agent_CHANGEME.pem',
            'share/temboard-agent_CHANGEME.key',
            'share/temboard-agent_ca_certs_CHANGEME.pem',
            'share/temboard-agent.logrotate'
        ]),
        ('lib/systemd/system', ['packaging/temboard-agent.service']),
    ])



import json
import os
import sys
from hashlib import md5
import datetime

from test.temboard import init_env, drop_env, temboard_request

# Import spc
tbda_dir = os.path.realpath(
            os.path.join(__file__, '..', '..'))

if tbda_dir not in sys.path:
    sys.path.insert(0, tbda_dir)

from temboardagent.spc import connector, error  # noqa

ENV = {}
XSESSION = ''


class TestPgconf(object):

    @classmethod
    def setup_class(cls):
        global ENV
        ENV = init_env()

    @classmethod
    def teardown_class(cls):
        drop_env(ENV)

    def _exec_query(self, dbname, query):
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database=dbname
        )
        conn.connect()
        conn.execute(query)
        conn.close()
        return list(conn.get_rows())

    def _read_file(self, filepath):
        try:
            with open(filepath, 'r') as fd:
                return fd.read()
        except Exception:
            pass

    def _temboard_login(self):
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/login' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={"Content-type": "application/json"},
                data={
                    'username': ENV['agent']['user'],
                    'password': ENV['agent']['password']
                    }
                )
        return json.loads(res)['session']

    def test_00_env_pg(self):
        """
        [pgconf] 00: PostgreSQL instance is up & running
        """
        conn = connector(
            host=ENV['pg']['socket_dir'],
            port=ENV['pg']['port'],
            user=ENV['pg']['user'],
            password=ENV['pg']['password'],
            database='postgres'
        )
        try:
            conn.connect()
            conn.close()
            global XSESSION
            XSESSION = self._temboard_login()
            assert True
        except error:
            assert False

    def test_01_get_pgconf_configuration_ok(self):
        """
        [pgconf] 01: GET /pgconf/configuration : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )

        # HTTP return code is 200.
        assert status == 200

        dict_data = json.loads(res)

        # Count the number of settings
        nb_settings = 0
        for cat in dict_data:
            nb_settings += len(cat['rows'])

        # dict_data length is equal to the number of category in pg_settings.
        assert len(dict_data) == self._exec_query(
                                    'postgres',
                                    "SELECT COUNT(DISTINCT(category)) AS nb "
                                    "FROM pg_settings")[0]['nb']
        # Total number of settings returned is equal to
        # the number of settings in pg_settings.
        assert nb_settings == self._exec_query(
                                'postgres',
                                "SELECT COUNT(*) AS nb "
                                "FROM pg_settings")[0]['nb']
        # Check first row's structure is right.
        assert 'context' in dict_data[0]['rows'][0]
        assert 'enumvals' in dict_data[0]['rows'][0]
        assert 'max_val' in dict_data[0]['rows'][0]
        assert 'vartype' in dict_data[0]['rows'][0]
        assert 'auto_val' in dict_data[0]['rows'][0]
        assert 'auto_val_raw' in dict_data[0]['rows'][0]
        assert 'boot_val' in dict_data[0]['rows'][0]
        assert 'unit' in dict_data[0]['rows'][0]
        assert 'desc' in dict_data[0]['rows'][0]
        assert 'name' in dict_data[0]['rows'][0]
        assert 'min_val' in dict_data[0]['rows'][0]
        assert 'setting' in dict_data[0]['rows'][0]
        assert 'setting_raw' in dict_data[0]['rows'][0]
        assert 'file_val' in dict_data[0]['rows'][0]
        assert 'file_val_raw' in dict_data[0]['rows'][0]
        assert len(dict_data[0]['rows'][0]) == 15

    def test_02_get_pgconf_configuration_ko_401(self):
        """
        [pgconf] 02: GET /pgconf/configuration : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_03_get_pgconf_configuration_ko_406(self):
        """
        [pgconf] 03: GET /pgconf/configuration : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_04_post_pgconf_configuration_ok(self):
        """
        [pgconf] 04: POST /pgconf/configuration : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'settings': [
                        {'name': 'autovacuum', 'setting': 'off'}
                    ]
                }
            )

        dict_data = json.loads(res)
        q = """
SELECT
    1 AS t
FROM
    regexp_split_to_table(pg_read_file('postgresql.auto.conf'),E'\n') AS l
WHERE
    l = 'autovacuum = ''off'''
"""
        res_query = self._exec_query('postgres', q)
        # HTTP return code is 200.
        assert status == 200
        # Check new 'autovacuum' value is in postgresql.auto.conf.
        assert len(res_query) > 0
        assert res_query[0]['t'] == 1
        # Check response's structure.
        assert 'settings' in dict_data
        assert type(dict_data['settings']) == list
        assert len(dict_data['settings']) == 1
        assert type(dict_data['settings'][0]) == dict
        assert 'setting' in dict_data['settings'][0]
        assert 'restart' in dict_data['settings'][0]
        assert 'name' in dict_data['settings'][0]
        assert 'previous_setting' in dict_data['settings'][0]
        assert dict_data['settings'][0]['name'] == 'autovacuum'
        assert dict_data['settings'][0]['setting'] == 'off'

    def test_05_post_pgconf_configuration_ko_401(self):
        """
        [pgconf] 05: POST /pgconf/configuration : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                },
                data={
                    'settings': [
                        {'name': 'autovacuum', 'setting': 'off'}
                    ]
                }
            )
        assert status == 401

    def test_06_get_pgconf_configuration_ko_406(self):
        """
        [pgconf] 06: POST /pgconf/configuration : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                },
                data={
                    'settings': [
                        {'name': 'autovacuum', 'setting': 'off'}
                    ]
                }
            )
        assert status == 406

    def test_07_post_pgconf_configuration_ko_400(self):
        """
        [pgconf] 07: POST /pgconf/configuration : HTTP return code is 400 on invalid json data
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/configuration' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data=''
            )
        assert status == 400

    def test_08_get_pgconf_configuration_status_ok(self):
        """
        [pgconf] 08: GET /pgconf/configuration/status : HTTP return code is 200 and the response data structure is right
        """  # noqa
        # shared_buffers update requires the server to restart
        self._exec_query(
                'postgres',
                "ALTER SYSTEM SET shared_buffers TO '256MB'"
            )
        # autovacuum update requires the server to reload
        self._exec_query(
                'postgres',
                "ALTER SYSTEM SET autovacuum TO 'on'"
            )
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/status' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data=''
            )
        dict_data = json.loads(res)
        # HTTP return code is 200.
        assert status == 200
        assert 'restart_pending' in dict_data
        assert 'restart_changes' in dict_data
        assert 'reload_pending' in dict_data
        assert 'reload_changes' in dict_data
        assert type(dict_data['restart_changes']) == list
        assert type(dict_data['reload_changes']) == list
        assert dict_data['restart_pending'] is True
        assert dict_data['reload_pending'] is True
        assert type(dict_data['restart_changes']) == list
        assert type(dict_data['reload_changes']) == list
        assert len(dict_data['restart_changes']) == 1
        assert len(dict_data['reload_changes']) == 1
        assert type(dict_data['restart_changes'][0]) == dict
        assert type(dict_data['reload_changes'][0]) == dict
        assert 'context' in dict_data['restart_changes'][0]
        assert 'enumvals' in dict_data['restart_changes'][0]
        assert 'max_val' in dict_data['restart_changes'][0]
        assert 'vartype' in dict_data['restart_changes'][0]
        assert 'auto_val' in dict_data['restart_changes'][0]
        assert 'auto_val_raw' in dict_data['restart_changes'][0]
        assert 'boot_val' in dict_data['restart_changes'][0]
        assert 'setting' in dict_data['restart_changes'][0]
        assert 'setting_raw' in dict_data['restart_changes'][0]
        assert 'file_val' in dict_data['restart_changes'][0]
        assert 'file_val_raw' in dict_data['restart_changes'][0]
        assert 'min_val' in dict_data['restart_changes'][0]
        assert 'pending_val' in dict_data['restart_changes'][0]
        assert 'unit' in dict_data['restart_changes'][0]
        assert 'desc' in dict_data['restart_changes'][0]
        assert 'name' in dict_data['restart_changes'][0]
        assert len(dict_data['restart_changes'][0]) == 16
        assert 'context' in dict_data['reload_changes'][0]
        assert 'enumvals' in dict_data['reload_changes'][0]
        assert 'max_val' in dict_data['reload_changes'][0]
        assert 'vartype' in dict_data['reload_changes'][0]
        assert 'setting' in dict_data['restart_changes'][0]
        assert 'setting_raw' in dict_data['restart_changes'][0]
        assert 'auto_val' in dict_data['reload_changes'][0]
        assert 'auto_val_raw' in dict_data['reload_changes'][0]
        assert 'boot_val' in dict_data['reload_changes'][0]
        assert 'file_val' in dict_data['reload_changes'][0]
        assert 'file_val_raw' in dict_data['reload_changes'][0]
        assert 'min_val' in dict_data['reload_changes'][0]
        assert 'pending_val' in dict_data['reload_changes'][0]
        assert 'unit' in dict_data['reload_changes'][0]
        assert 'desc' in dict_data['reload_changes'][0]
        assert 'name' in dict_data['reload_changes'][0]
        assert len(dict_data['reload_changes'][0]) == 16

    def test_09_get_pgconf_configuration_status_ko_406(self):
        """
        [pgconf] 09: GET /pgconf/configuration/status : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/status' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_10_get_pgconf_configuration_status_ko_401(self):
        """
        [pgconf] 10: GET /pgconf/configuration/status : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/status' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_11_get_pgconf_configuration_categories(self):
        """
        [pgconf] 11: GET /pgconf/configuration/categories : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/categories' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        assert status == 200
        assert 'categories' in dict_data
        assert type(dict_data['categories']) == list
        assert len(dict_data['categories']) == \
            self._exec_query(
                'postgres',
                "SELECT COUNT(DISTINCT(category)) AS nb "
                "FROM pg_settings")[0]['nb']
        assert dict_data['categories'][0] == \
            self._exec_query(
                'postgres',
                "SELECT category FROM pg_settings "
                "ORDER BY category LIMIT 1")[0]['category']

    def test_12_get_pgconf_configuration_categories_ko_401(self):
        """
        [pgconf] 12: GET /pgconf/configuration/categories : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/categories' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_13_get_pgconf_configuration_categories_ko_406(self):
        """
        [pgconf] 13: GET /pgconf/configuration/categories : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/categories' % (
                    ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_14_get_pgconf_configuration_category_ok(self):
        """
        [pgconf] 14: GET /pgconf/configuration/category/:categoryname : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/category/Autovacuum'
                    % (ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)

        # Count the number of settings
        nb_settings = 0
        for cat in dict_data:
            nb_settings += len(cat['rows'])

        assert status == 200
        assert len(dict_data) == 1
        assert nb_settings == self._exec_query(
                                'postgres',
                                "SELECT COUNT(*) AS nb FROM pg_settings "
                                "WHERE category = 'Autovacuum'")[0]['nb']
        assert 'context' in dict_data[0]['rows'][0]
        assert 'enumvals' in dict_data[0]['rows'][0]
        assert 'max_val' in dict_data[0]['rows'][0]
        assert 'vartype' in dict_data[0]['rows'][0]
        assert 'auto_val' in dict_data[0]['rows'][0]
        assert 'auto_val_raw' in dict_data[0]['rows'][0]
        assert 'boot_val' in dict_data[0]['rows'][0]
        assert 'unit' in dict_data[0]['rows'][0]
        assert 'desc' in dict_data[0]['rows'][0]
        assert 'name' in dict_data[0]['rows'][0]
        assert 'min_val' in dict_data[0]['rows'][0]
        assert 'setting' in dict_data[0]['rows'][0]
        assert 'setting_raw' in dict_data[0]['rows'][0]
        assert 'file_val' in dict_data[0]['rows'][0]
        assert 'file_val_raw' in dict_data[0]['rows'][0]
        assert len(dict_data[0]['rows'][0]) == 15

    def test_15_get_pgconf_configuration_category_ko_401(self):
        """
        [pgconf] 15: GET /pgconf/configuration/category/:categoryname : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/category/Autovacuum'
                    % (ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_16_get_pgconf_configuration_category_ko_406(self):
        """
        [pgconf] 16: GET /pgconf/configuration/category/:categoryname : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/configuration/category/Autovacuum'
                    % (ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_17_get_pgconf_hba_ok(self):
        """
        [pgconf] 17: GET /pgconf/hba : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'version' in dict_data
        assert 'filepath' in dict_data
        assert dict_data['version'] is None
        assert type(dict_data['filepath']) == unicode
        assert 'entries' in dict_data
        assert len(dict_data['entries']) > 0
        for row in dict_data['entries']:
            if 'comment' not in row:
                assert 'connection' in row
                assert 'database' in row
                assert 'user' in row
                assert 'address' in row
                assert 'auth_method' in row
                assert 'auth_options' in row
            else:
                assert 'comment' in row

    def test_18_get_pgconf_hba_ko_401(self):
        """
        [pgconf] 18: GET /pgconf/hba : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_19_get_pgconf_hba_ko_406(self):
        """
        [pgconf] 19: GET /pgconf/hba : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_20_get_pgconf_hba_raw_ok(self):
        """
        [pgconf] 20: GET /pgconf/hba/raw : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        md5sum_hba = self._exec_query(
                        'postgres',
                        "SELECT md5(pg_read_file('pg_hba.conf')) "
                        "AS md5sum_hba")[0]['md5sum_hba']
        assert status == 200
        assert 'version' in dict_data
        assert 'filepath' in dict_data
        assert dict_data['version'] is None
        assert type(dict_data['filepath']) == unicode
        assert 'content' in dict_data
        assert md5(dict_data['content']).hexdigest() == md5sum_hba

    def test_21_get_pgconf_hba_raw_ko_401(self):
        """
        [pgconf] 21: GET /pgconf/hba/raw : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_22_get_pgconf_hba_raw_ko_406(self):
        """
        [pgconf] 22: GET /pgconf/hba/raw : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "X" * 12
                }
            )
        assert status == 406

    def test_23_post_pgconf_hba_ok(self):
        """
        [pgconf] 23: POST /pgconf/hba : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'entries':
                    [
                        {
                            'connection': 'local',
                            'database': 'all',
                            'user': 'all',
                            'auth_method': 'trust',
                            'auth_options': 'blahblah'
                        },
                        {
                            'connection': 'host',
                            'database': 'test',
                            'user': 'test',
                            'address': '192.168.1.0/24',
                            'auth_method': 'trust'
                        },
                        {
                            'connection': 'hostssl',
                            'database': 'test2',
                            'user': 'test2',
                            'address': '192.168.1.0 255.255.255.0',
                            'auth_method': 'trust'
                        }
                    ]
                }
            )
        dict_data = json.loads(res)
        md5sum_hba = self._exec_query(
                        'postgres',
                        "SELECT md5(pg_read_file('pg_hba.conf')) "
                        "AS md5sum_hba")[0]['md5sum_hba']
        # Expected hba raw content.
        exp_hba_raw = "local  all  all  trust blahblah\r\nhost  test  test 192.168.1.0/24 trust \r\nhostssl  test2  test2 192.168.1.0 255.255.255.0 trust \r\n"  # noqa
        assert status == 200
        assert type(dict_data) == dict
        assert 'last_version' in dict_data
        assert 'filepath' in dict_data
        assert md5sum_hba == md5(exp_hba_raw).hexdigest()

    def test_24_post_pgconf_hba_ko_401(self):
        """
        [pgconf] 24: POST /pgconf/hba : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                },
                data={
                    'entries':
                    [
                        {
                            'connection': 'local',
                            'database': 'all',
                            'user': 'all',
                            'auth_method': 'trust',
                            'auth_options': 'blahblah'
                        }
                    ]
                }
            )
        assert status == 401

    def test_25_post_pgconf_hba_ko_406(self):
        """
        [pgconf] 25: POST /pgconf/hba : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                },
                data={
                    'entries':
                    [
                        {
                            'connection': 'local',
                            'database': 'all',
                            'user': 'all',
                            'auth_method': 'trust',
                            'auth_options': 'blahblah'
                        }
                    ]
                }
            )
        assert status == 406

    def test_26_post_pgconf_hba_ko_406(self):
        """
        [pgconf] 26: POST /pgconf/hba : HTTP return code is 406 on invalid data
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                },
                data={'toto': 'tutu'}
            )
        assert status == 406

    def test_27_post_pgconf_hba_version_ok(self):
        """
        [pgconf] 27: POST /pgconf/hba : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'entries':
                    [
                        {
                            'connection': 'local',
                            'database': 'all',
                            'user': 'all',
                            'auth_method': 'trust'
                        }
                    ],
                    'new_version': True
                }
            )
        dict_data = json.loads(res)
        md5sum_hba = self._exec_query(
                        'postgres',
                        "SELECT md5(pg_read_file('pg_hba.conf')) "
                        "AS md5sum_hba")[0]['md5sum_hba']
        # Expected hba raw content.
        exp_hba_raw = "local  all  all  trust \r\n"

        assert status == 200
        assert type(dict_data) == dict
        assert 'last_version' in dict_data
        try:
            datetime.datetime.strptime(
                            dict_data['last_version'],
                            "%Y-%m-%dT%H:%M:%S")
        except ValueError:
            assert False
        assert 'filepath' in dict_data
        assert md5sum_hba == md5(exp_hba_raw).hexdigest()

    def test_28_get_pgconf_hba_versions_ok(self):
        """
        [pgconf] 28: GET /pgconf/hba/versions: HTTP return code is 200 and the repsonse date structure is right.
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'filepath' in dict_data
        assert 'versions' in dict_data
        assert type(dict_data['versions']) == list
        assert len(dict_data['versions']) == 1
        assert os.path.isfile("%s.%s" % (
                dict_data['filepath'],
                dict_data['versions'][0]))

    def test_29_get_pgconf_hba_versions_ko_401(self):
        """
        [pgconf] 29: GET /pgconf/hba/versions: HTTP return code is 401 on invalid xsession.
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_30_get_pgconf_hba_versions_ko_406(self):
        """
        [pgconf] 30: GET /pgconf/hba/versions: HTTP return code is 406 on malformed xsession.
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "XXXXXXXXXXX"
                }
            )
        assert status == 406

    def test_31_get_pgconf_hba_version_ok(self):
        """
        [pgconf] 31: GET /pgconf/hba?version=:version : HTTP return code is 200 and the response data structure is right
        """  # noqa
        # We need to get the last version id.
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'filepath' in dict_data
        assert 'versions' in dict_data
        assert type(dict_data['versions']) == list
        assert len(dict_data['versions']) == 1
        assert os.path.isfile("%s.%s" % (
            dict_data['filepath'],
            dict_data['versions'][0]
            )
        )
        last_version = dict_data['versions'][0]
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        last_version
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'version' in dict_data
        assert 'filepath' in dict_data
        assert dict_data['version'] == last_version
        assert type(dict_data['filepath']) == unicode
        assert 'entries' in dict_data
        assert len(dict_data['entries']) > 0
        assert 'connection' in dict_data['entries'][0]
        assert 'database' in dict_data['entries'][0]
        assert 'user' in dict_data['entries'][0]
        assert 'address' in dict_data['entries'][0]
        assert 'auth_method' in dict_data['entries'][0]
        assert 'auth_options' in dict_data['entries'][0]

    def test_32_get_pgconf_hba_version_ko_404(self):
        """
        [pgconf] 32: GET /pgconf/hba?version=:version : HTTP return code is 404 on unexisting version
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba?version=1970-01-01T00:00:01' %
                    (ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 404

    def test_33_get_pgconf_hba_version_ko_406(self):
        """
        [pgconf] 33: GET /pgconf/hba?version=:version : HTTP return code is 406 on malformed version id
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba?version=XX112233' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 406

    def test_34_get_pgconf_hba_raw_version_ok(self):
        """
        [pgconf] 34: GET /pgconf/hba/raw?version=:version : HTTP return code is 200 and the response data structure is right
        """  # noqa
        # We need to get the last version id.
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'], ENV['agent']['port']),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'filepath' in dict_data
        assert 'versions' in dict_data
        assert type(dict_data['versions']) == list
        assert len(dict_data['versions']) == 1
        assert os.path.isfile("%s.%s" % (
                dict_data['filepath'],
                dict_data['versions'][0]
                )
            )
        last_version = dict_data['versions'][0]
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        last_version
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'version' in dict_data
        assert 'filepath' in dict_data
        assert dict_data['version'] == last_version
        assert type(dict_data['filepath']) == unicode
        assert 'content' in dict_data
        assert md5(dict_data['content']).hexdigest() == \
            md5(self._read_file("%s.%s" % (
                dict_data['filepath'],
                last_version)
                )).hexdigest()

    def test_35_get_pgconf_hba_raw_version_ko_404(self):
        """
        [pgconf] 35: GET /pgconf/hba/raw?version=:version : HTTP return code is 404 on unexisting version
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        '1970-01-01T00:00:01'
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 404

    def test_36_get_pgconf_hba_raw_version_ko_406(self):
        """
        [pgconf] 36: GET /pgconf/hba/raw?version=:version : HTTP return code is 406 on malformed version id
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/raw?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        'XX112233'
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 406

    def test_37_delete_pgconf_hba_version_ok(self):
        """
        [pgconf] 37: DELETE /pgconf/hba/raw?version=:version : HTTP return code is 200 and data structure is right
        """  # noqa
        # We need to get the last version id.
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/hba/versions' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'filepath' in dict_data
        assert 'versions' in dict_data
        assert type(dict_data['versions']) == list
        assert len(dict_data['versions']) == 1
        assert os.path.isfile("%s.%s" % (
                dict_data['filepath'],
                dict_data['versions'][0]
                )
            )
        last_version = dict_data['versions'][0]
        filepath = dict_data['filepath']
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='DELETE',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        last_version
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        assert status == 200
        assert 'deleted' in dict_data
        assert 'version' in dict_data
        assert dict_data['version'] == last_version
        assert dict_data['deleted'] is True
        assert not os.path.isfile("%s.%s" % (filepath, last_version))

    def test_38_delete_pgconf_hba_version_ko_401(self):
        """
        [pgconf] 38: DELETE /pgconf/hba/raw?version=:version : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='DELETE',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        '1970-01-01T00:00:01'
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_39_delete_pgconf_hba_version_ko_406(self):
        """
        [pgconf] 39: DELETE /pgconf/hba/raw?version=:version : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='DELETE',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        '1970-01-01T00:00:01'
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 12
                }
            )
        assert status == 406

    def test_40_delete_pgconf_hba_version_ko_406(self):
        """
        [pgconf] 40: DELETE /pgconf/hba/raw?version=:version : HTTP return code is 406 on malformed version id
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='DELETE',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        'A' * 6
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 406

    def test_41_delete_pgconf_hba_version_ko_404(self):
        """
        [pgconf] 41: DELETE /pgconf/hba/raw?version=:version : HTTP return code is 404 on unexisting version
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='DELETE',
                url='https://%s:%s/pgconf/hba?version=%s' % (
                        ENV['agent']['host'],
                        ENV['agent']['port'],
                        '1970-01-01T00:00:01'
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        assert status == 404

    def test_42_get_pgconf_pg_ident_ok(self):
        """
        [pgconf] 42: GET /pgconf/pg_ident : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                }
            )
        dict_data = json.loads(res)
        md5sum_pg_ident = self._exec_query(
                            'postgres',
                            "SELECT md5(pg_read_file('pg_ident.conf')) "
                            "AS md5sum_pg_ident")[0]['md5sum_pg_ident']
        assert status == 200
        assert 'filepath' in dict_data
        assert type(dict_data['filepath']) == unicode
        assert 'content' in dict_data
        assert md5(dict_data['content']).hexdigest() == md5sum_pg_ident

    def test_43_get_pgconf_pg_ident_ko_401(self):
        """
        [pgconf] 43: GET /pgconf/pg_ident : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                }
            )
        assert status == 401

    def test_44_get_pgconf_pg_ident_ko_406(self):
        """
        [pgconf] 44: GET /pgconf/pg_ident : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='GET',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "X" * 12
                }
            )
        assert status == 406

    def test_45_post_pgconf_hba_raw_ok(self):
        """
        [pgconf] 45: POST /pgconf/hba/raw : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'content': 'local all all trust',
                    'new_version': True
                }
            )
        dict_data = json.loads(res)
        md5sum_hba = self._exec_query(
                        'postgres',
                        "SELECT md5(pg_read_file('pg_hba.conf')) "
                        "AS md5sum_hba")[0]['md5sum_hba']
        # Expected hba raw content.
        exp_hba_raw = "local all all trust"

        assert status == 200
        assert type(dict_data) == dict
        assert 'last_version' in dict_data
        try:
            datetime.datetime.strptime(
                            dict_data['last_version'],
                            "%Y-%m-%dT%H:%M:%S")
        except ValueError:
            assert False
        assert 'filepath' in dict_data
        assert md5sum_hba == md5(exp_hba_raw).hexdigest()

    def test_46_post_pgconf_hba_raw_ko_401(self):
        """
        [pgconf] 46: POST /pgconf/hba/raw : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                },
                data={
                    'content': 'local all all trust',
                    'new_version': True
                }
            )
        assert status == 401

    def test_47_post_pgconf_hba_raw_ko_406(self):
        """
        [pgconf] 47: POST /pgconf/hba/raw : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "X" * 12
                },
                data={
                    'content': 'local all all trust',
                    'new_version': True
                }
            )
        assert status == 406

    def test_48_post_pgconf_hba_raw_ko_406(self):
        """
        [pgconf] 48: POST /pgconf/hba/raw : HTTP return code is 406 on invalid data
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/hba/raw' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'toto': 'tutu'
                }
            )
        assert status == 406

    def test_49_post_pgconf_pg_ident_ok(self):
        """
        [pgconf] 49: POST /pgconf/pg_ident : HTTP return code is 200 and the response data structure is right
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'content': '# test\r\n# test\r\n'
                }
            )
        dict_data = json.loads(res)
        md5sum_pg_ident = self._exec_query(
                            'postgres',
                            "SELECT md5(pg_read_file('pg_ident.conf')) AS "
                            "md5sum_pg_ident")[0]['md5sum_pg_ident']
        # Expected hba raw content.
        exp_pg_ident_raw = "# test\r\n# test\r\n"

        assert status == 200
        assert type(dict_data) == dict
        assert 'update' in dict_data
        assert dict_data['update'] is True
        assert md5sum_pg_ident == md5(exp_pg_ident_raw).hexdigest()

    def test_50_post_pgconf_pg_ident_ko_401(self):
        """
        [pgconf] 50: POST /pgconf/pg_ident : HTTP return code is 401 on invalid xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "3" * 64
                },
                data={
                    'content': '# test\r\n# test\r\n'
                }
            )
        assert status == 401

    def test_51_post_pgconf_pg_ident_ko_406(self):
        """
        [pgconf] 51: POST /pgconf/pg_ident : HTTP return code is 406 on malformed xsession
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": "X" * 12
                },
                data={
                    'content': '# test\r\n# test\r\n'
                }
            )
        assert status == 406

    def test_52_post_pgconf_pg_ident_ko_406(self):
        """
        [pgconf] 52: POST /pgconf/pg_ident : HTTP return code is 406 on invalid data
        """  # noqa
        (status, res) = temboard_request(
                ENV['agent']['ssl_cert_file'],
                method='POST',
                url='https://%s:%s/pgconf/pg_ident' % (
                        ENV['agent']['host'],
                        ENV['agent']['port']
                    ),
                headers={
                    "Content-type": "application/json",
                    "X-Session": XSESSION
                },
                data={
                    'toto': 'tutu'
                }
            )
        assert status == 406
